@@ -46,8 +46,8 @@ extract_environment <- function(x, env, value = TRUE, markup = c("latex", "markd
 extract_command <- function(x, command, type = c("character", "logical", "numeric"), markup = c("latex", "markdown"))
 {
   ## return type and markup type
-  type <- match.arg(type)
-  markup <- match.arg(markup)
+  type <- match.arg(type, c("character", "logical", "numeric"))
+  markup <- match.arg(markup, c("latex", "markdown"))
 
   ## find command line
   command <- if(markup == "latex") paste0("\\", command) else paste0(command, ":")
@@ -73,6 +73,11 @@ extract_command <- function(x, command, type = c("character", "logical", "numeri
     rval <- gsub("[ \t]+$", "", rval)
     ## split further with respect to other symbols (currently only |)
     rval <- unlist(strsplit(rval, "|", fixed = TRUE))
+    ## translate HTML scientific notation produced by knitr
+    if(type == "numeric" && any(grepl("\\times 10^", x, fixed = TRUE) & grepl("\\ensuremath", x, fixed = TRUE))) {
+      rval <- gsub("\\ensuremath", "", rval, fixed = TRUE)
+      rval <- gsub("([ \t]*\\\\times[ \t]*10\\^)([-]*[0-9]+)", "e\\2", rval)
+    }
   } else {
     ## strip off command
     rval <- gsub(command, "", rval, fixed = TRUE)
@@ -81,6 +86,10 @@ extract_command <- function(x, command, type = c("character", "logical", "numeri
     rval <- gsub("[ \t]+$", "", rval)
     ## split further with respect to other symbols (currently only |)
     rval <- unlist(strsplit(rval, "|", fixed = TRUE))
+    ## translate HTML scientific notation produced by knitr
+    if(type == "numeric" && any(grepl("&times; 10<sup>", x, fixed = TRUE))) {
+      rval <- gsub("([ \t]*&times;[ \t]*10<sup>)([-]*[0-9]+)(</sup>)", "e\\2", rval)
+    }
   }
 
   ## convert to return type
@@ -90,7 +99,7 @@ extract_command <- function(x, command, type = c("character", "logical", "numeri
 extract_extra <- function(x, markup = c("latex", "markdown"))
 {
   ## markup type
-  markup <- match.arg(markup)
+  markup <- match.arg(markup, c("latex", "markdown"))
 
   ## search for extra commands
   comm0 <- if(markup == "latex") "\\exextra[" else "exextra["
@@ -115,12 +124,11 @@ extract_extra <- function(x, markup = c("latex", "markdown"))
 extract_items <- function(x, markup = c("latex", "markdown"))
 {
   ## markup type
-  markup <- match.arg(markup)
+  markup <- match.arg(markup, c("latex", "markdown"))
 
   ## map markdown to tex
   if(markup == "markdown") {
-    x <- gsub("^\\* ", "\\\\item ", x)
-    x <- gsub("^- ", "\\\\item ", x)
+    x <- gsub("^[\\*|-][[:space:]]*", "\\\\item ", x)
     x[x %in% c("*", "-")] <- "\\item "    
   }
     
@@ -153,6 +161,10 @@ read_metainfo <- function(file, markup = NULL, exshuffle = NULL)
     "rmd"  = "markdown"
   )
 
+  ## only read meta-information from corresponding environment/section in Rmd files
+  ## (in Rnw files there is no meta-information environment)
+  if(markup == "markdown") x <- extract_environment(x, "metainformation", markup = markup)
+
   ## overwrite exshuffle settings?
   exshuffle_overwrite <- !is.null(exshuffle)
   exshuffle_arg <- exshuffle
@@ -163,12 +175,15 @@ read_metainfo <- function(file, markup = NULL, exshuffle = NULL)
   exname <- extract_command(x, "exname", markup = markup)            ## short name/description, only to be used for printing within R
   extitle <- extract_command(x, "extitle", markup = markup)          ## pretty longer title
   exsection <- extract_command(x, "exsection", markup = markup)      ## sections for groups of exercises, use slashes for subsections (like URL)
+  extags <- extract_command(x, "extags", markup = markup)            ## arbitrary individual tags, can contain "=" for tag classes/groups
+  exauthor <- extract_command(x, "exauthor", markup = markup)        ## author of exercise
   exversion <- extract_command(x, "exversion", markup = markup)      ## version of exercise
 
   ## Question & Solution ###########################
   exsolution <- extract_command(x, "exsolution", markup = markup)    ## solution, valid values depend on extype
   extol <- extract_command(x, "extol", "numeric", markup = markup)   ## optional tolerance limit for numeric solutions
   exclozetype <- extract_command(x, "exclozetype", markup = markup)  ## type of individual cloze solutions
+  exstringtype <- extract_command(x, "exstringtype", markup = markup)## essay+file
 
   ## E-Learning & Exam ###################################
   expoints     <- extract_command(x, "expoints",    "numeric", markup = markup)   ## default points
@@ -176,7 +191,6 @@ read_metainfo <- function(file, markup = NULL, exshuffle = NULL)
   exshuffle    <- extract_command(x, "exshuffle",   "character", markup = markup) ## shuffle schoice/mchoice answers?
   exmaxchars   <- extract_command(x, "exmaxchars",   markup = markup)             ## maximum number of characters in string answers
   exabstention <- extract_command(x, "exabstention", markup = markup)             ## string for abstention in schoice/mchoice answers
-  exstringtype <- extract_command(x, "exstringtype", markup = markup)             ## essay+file
 
   ## User-Defined ###################################
   exextra <- extract_extra(x, markup = markup)
@@ -198,7 +212,7 @@ read_metainfo <- function(file, markup = NULL, exshuffle = NULL)
   slength <- length(exsolution)
   if(slength < 1L) stop("no exsolution specified")
   exsolution <- switch(extype,
-    "schoice" = string2mchoice(exsolution, single = !is.numeric(exshuffle)), ## check for _single_ choice (unless sub-shuffling afterwards)
+    "schoice" = string2mchoice(exsolution, single = !is.numeric(exshuffle) && !exshuffle_overwrite), ## check for _single_ choice (unless sub-shuffling afterwards)
     "mchoice" = string2mchoice(exsolution),
     "num" = string2num(exsolution),
     "string" = exsolution,
@@ -311,26 +325,31 @@ read_metainfo <- function(file, markup = NULL, exshuffle = NULL)
       exmaxchars <- exmaxchars[[1]]
   }
 
-  ## return everything (backward compatible with earlier versions)
+  ## return everything
   rval <- list(
     file = tools::file_path_sans_ext(file),
     markup = markup,
+
     type = extype,
     name = exname,
     title = extitle,
     section = exsection,
+    tags = extags,
+    author = exauthor,
     version = exversion,
+
     solution = exsolution,
     tolerance = extol,
     clozetype = exclozetype,
+    stringtype = exstringtype,
+
     points = expoints,
     time = extime,
     shuffle = exshuffle,
     length = slength,
     string = string,
     maxchars = exmaxchars,
-    abstention = exabstention,
-    stringtype = exstringtype
+    abstention = exabstention
   )
   rval <- c(rval, exextra)
   return(rval)
